package org.eclipse.jdt.internal.compiler.util;

package .tmp;

import java.io.OutputStream;
import java.io.PrintWriter;
import java.io.Writer;
import java.util.Arrays;
import java.util.Comparator;
import java.util.HashMap;
import java.util.Map;

public class GenericXMLWriter extends PrintWriter {
  private static final String XML_VERSION = "<?xml version=\"1.0\" encoding=\"UTF-8\"?>";
  
  private String lineSeparator;
  
  private int tab;
  
  private static void appendEscapedChar(StringBuffer buffer, char c) {
    String replacement = getReplacement(c);
    if (replacement != null) {
      buffer.append('&');
      buffer.append(replacement);
      buffer.append(';');
    } else {
      buffer.append(c);
    } 
  }
  
  private static String getEscaped(String s) {
    StringBuffer result = new StringBuffer(s.length() + 10);
    for (int i = 0; i < s.length(); i++)
      appendEscapedChar(result, s.charAt(i)); 
    return result.toString();
  }
  
  private static String getReplacement(char c) {
    switch (c) {
      case '<':
        return "lt";
      case '>':
        return "gt";
      case '"':
        return "quot";
      case '\'':
        return "apos";
      case '&':
        return "amp";
    } 
    return null;
  }
  
  public GenericXMLWriter(OutputStream stream, String lineSeparator, boolean printXmlVersion) {
    this(new PrintWriter(stream), lineSeparator, printXmlVersion);
  }
  
  public GenericXMLWriter(Writer writer, String lineSeparator, boolean printXmlVersion) {
    super(writer);
    this.tab = 0;
    this.lineSeparator = lineSeparator;
    if (printXmlVersion) {
      print("<?xml version=\"1.0\" encoding=\"UTF-8\"?>");
      print(this.lineSeparator);
    } 
  }
  
  public void endTag(String name, boolean insertTab, boolean insertNewLine) {
    this.tab--;
    printTag(String.valueOf('/') + name, null, insertTab, insertNewLine, false);
  }
  
  public void printString(String string, boolean insertTab, boolean insertNewLine) {
    if (insertTab)
      printTabulation(); 
    print(string);
    if (insertNewLine)
      print(this.lineSeparator); 
  }
  
  private void printTabulation() {
    for (int i = 0; i < this.tab; ) {
      print('\t');
      i++;
    } 
  }
  
  public void printTag(String name, HashMap parameters, boolean insertTab, boolean insertNewLine, boolean closeTag) {
    if (insertTab)
      printTabulation(); 
    print('<');
    print(name);
    if (parameters != null) {
      int length = parameters.size();
      Map.Entry[] entries = new Map.Entry[length];
      parameters.entrySet().toArray((Object[])entries);
      Arrays.sort(entries, (Comparator<? super Map.Entry>)new Object(this));
      for (int i = 0; i < length; i++) {
        print(' ');
        print(entries[i].getKey());
        print("=\"");
        print(getEscaped(String.valueOf(entries[i].getValue())));
        print('"');
      } 
    } 
    if (closeTag) {
      print("/>");
    } else {
      print(">");
    } 
    if (insertNewLine)
      print(this.lineSeparator); 
    if (parameters != null && !closeTag)
      this.tab++; 
  }
  
  public void startTag(String name, boolean insertTab) {
    printTag(name, null, insertTab, true, false);
    this.tab++;
  }
}
